pub struct NonceManager {
pub nonces: <Mapping<Address, Mapping<U256, u64>> as StorableType>::Handler,
pub expiring_nonce_seen: <Mapping<B256, u64> as StorableType>::Handler,
pub expiring_nonce_ring: <Mapping<u32, B256> as StorableType>::Handler,
pub expiring_nonce_ring_ptr: <u32 as StorableType>::Handler,
address: Address,
storage: StorageCtx,
}Fields
nonces: <Mapping<Address, Mapping<U256, u64>> as StorableType>::Handlerexpiring_nonce_seen: <Mapping<B256, u64> as StorableType>::Handlerexpiring_nonce_ring: <Mapping<u32, B256> as StorableType>::Handlerexpiring_nonce_ring_ptr: <u32 as StorableType>::Handleraddress: Addressstorage: StorageCtxImplementations
Sourceimpl NonceManager
impl NonceManager
Sourcepub fn new() -> Self
pub fn new() -> Self
Creates an instance of the precompile.
Caution: This does not initialize the account, see Self::initialize.
fn __new(address: Address) -> Self
fn __initialize(&mut self) -> Result<()>
fn emit_event(&mut self, event: impl IntoLogData) -> Result<()>
pub fn emitted_events(&self) -> &Vec<LogData>
test-utils only.pub fn clear_emitted_events(&mut self)
test-utils only.pub fn assert_emitted_events(&self, expected: Vec<impl IntoLogData>)
test-utils only.Sourceimpl NonceManager
impl NonceManager
Sourcepub fn initialize(&mut self) -> Result<()>
pub fn initialize(&mut self) -> Result<()>
Initializes the nonce manager precompile storage layout.
Sourcepub fn get_nonce(&self, call: getNonceCall) -> Result<u64>
pub fn get_nonce(&self, call: getNonceCall) -> Result<u64>
Returns the current nonce for account at the given nonceKey.
Errors
ProtocolNonceNotSupported— nonce key 0 is the protocol nonce and cannot be read here
Sourcepub fn increment_nonce(
&mut self,
account: Address,
nonce_key: U256,
) -> Result<u64>
pub fn increment_nonce( &mut self, account: Address, nonce_key: U256, ) -> Result<u64>
Increments the 2D nonce for account at nonce_key and returns the new value, enabling
concurrent transaction execution. Key 0 is reserved for the protocol nonce.
Errors
InvalidNonceKey—nonce_keyis 0, which is reserved for the protocol nonceNonceOverflow— the current nonce value isu64::MAXand cannot be incremented
Sourcepub fn is_expiring_nonce_seen(&self, hash: B256, now: u64) -> Result<bool>
pub fn is_expiring_nonce_seen(&self, hash: B256, now: u64) -> Result<bool>
Checks if a hash has been seen and is still valid (not expired). NOTE: internally used by the transaction pool.
Sourcepub fn check_and_mark_expiring_nonce(
&mut self,
expiring_nonce_hash: B256,
valid_before: u64,
) -> Result<()>
pub fn check_and_mark_expiring_nonce( &mut self, expiring_nonce_hash: B256, valid_before: u64, ) -> Result<()>
Validates and records an expiring nonce transaction. Uses a
circular buffer that overwrites expired entries as the pointer
advances. The hash is keccak256(encode_for_signing || sender),
invariant to fee payer changes.
Uses a circular buffer that overwrites expired entries as the pointer advances.
The expiring_nonce_hash parameter is
(keccak256(encode_for_signing || sender)), which is invariant to fee payer changes.
This is called during transaction execution to:
- Validate the expiry is within the allowed window
- Check for replay (hash already seen and not expired)
- Check if we can evict the entry at current pointer (must be expired or empty)
- Mark the hash as seen
Errors
InvalidExpiringNonceExpiry—valid_beforenot in (now, now + EXPIRING_NONCE_MAX_EXPIRY_SECS]ExpiringNonceReplay— transaction hash is already recorded and has not yet expiredExpiringNonceSetFull— the circular buffer slot holds an unexpired entry that can’t be evicted
Sourceimpl NonceManager
impl NonceManager
Sourcepub fn create_precompile(cfg: &CfgEnv<TempoHardfork>) -> DynPrecompile
pub fn create_precompile(cfg: &CfgEnv<TempoHardfork>) -> DynPrecompile
Creates the EVM precompile for this type.